@@ -31,6 +31,8 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 struct wl_buffer;
 struct wl_surface;
 
+class QWindow;
+
 namespace KWayland
 {
 namespace Client
@@ -58,6 +60,17 @@ public:
     explicit Surface(QObject *parent = nullptr);
     virtual ~Surface();
 
+    /**
+     * Creates a Surface for the given @p window.
+     * This is an integration feature for QtWayland. On non-wayland platforms this method returns
+     * @c nullptr as well as for not created QWindows.
+     *
+     * The returned Surface will be fully setup, but won't be released. It gets automatically
+     * destroyed together with the @p window.
+     * @since 5.4
+     **/
+    static Surface *fromWindow(QWindow *window);
+
     /**
      * Setup this Surface to manage the @p surface.
      * When using Compositor::createSurface there is no need to call this
@@ -180,6 +193,12 @@ public:
     operator wl_surface*();
     operator wl_surface*() const;
 
+    /**
+     * @returns the id of the referenced wl_proxy.
+     * @since 5.4
+     **/
+    quint32 id() const;
+
     /**
      * All Surfaces which are currently created.
      * TODO: KF6 return QList<Surface*> instead of const-ref
